%% Main function of the DCT-based method for testing images in "MPEG7 dataset"
% lossy contour compression
clc; clear; close all
path = '../MPEG7 dataset/';
addpath('../Common functions');
FileList=dir([path '*.gif']);
%% Choose one image
choose=1;   % from 1 to 47

if choose<1 || choose > length(FileList)
    error(['Image index out of bound! "choose" should be from 1 to ' num2str(length(FileList))]);
end

%% Coordinates sequence of image
ImageName = [path, FileList(choose).name];
img=imread(ImageName);

if size(img,3)==3
   img = rgb2gray(imread(ImageName));
end
img=im2bw(img,graythresh(img));
img=double(img);
[M,N]=size(img);

Edge = bwboundaries(img,8,'noholes');
cross=[];
OpenFlag=zeros(1,length(Edge));

for i=1:length(Edge)
    edgey{i} = Edge{i}(:,1)'; % y coordinate of edge
    edgex{i} = Edge{i}(:,2)'; % x coordinate of edge
end

%% Encoding

% parameter for the DCT-based method
q=2;  % quantization factor
pn=3;   % the number of coefficients retaining for the DCT
eth=20; % error threshold

tic;
[Code, x_apx, y_apx, DPx, DPy] = DCT_encode(edgex, edgey, M, N, OpenFlag, cross, pn, eth, q);
encoding_time=toc;
disp(['Encoding time: ' num2str(encoding_time) 'seconds.']);


%% Decoding
tic
[x_apx_r, y_apx_r, DPx_r, DPy_r] = DCT_decode(Code, M, N, pn, q);    % _r for recovery
decoding_time=toc;
disp(['Decoding time: ' num2str(decoding_time) 'seconds.']);

%% Evaluation
[DP, ISE, MSE, CR] = evaluate2(DPx, DPy, x_apx_r, y_apx_r, edgex, edgey);
disp(['Number of DP: ' num2str(DP)]);
disp(['Integral Squared Error: ' num2str(ISE)]);
disp(['Mean Squared Error: ' num2str(MSE)]);
disp(['Compression Ratio: ' num2str(CR)]);
disp(['Coding Length: ' num2str(length(Code))]);

%% Plot
Nc=length(edgex);
figure('Units','normalized','position',[.2 .2 N/max(M,N)/2 M/max(M,N)/2]);
subplot(1,2,1); hold on
title('The Original Contour');
for i=1:Nc
    plot(edgex{i},edgey{i},'b');
end
set(gca,'Ydir','reverse','XTickLabel',[],'YTickLabel',[]); axis([1,N,1,M]);
% legend('Original Contour','location','best');
hold off

subplot(1,2,2); hold on
title('The Approximated Contour');
for i=1:Nc
    plot(x_apx_r{i},y_apx_r{i},'k',DPx_r{i}, DPy_r{i},'ro'); 
end
set(gca,'Ydir','reverse','XTickLabel',[],'YTickLabel',[]); axis([1,N,1,M]);
legend('Approximated Contour','Dominant Points','location','best');
hold off