%% Main function of the DCT-based method for testing images in "Zalik dataset"
% lossy contour compression
clc; clear; close all
path = '../Zalik dataset/';
addpath('../Common functions');
%% Choose one image
choose =1;
switch choose
        case 1
            name = 'bird';
        case 2
            name = 'circle';
        case 3
            name = 'butterfly';
        case 4 
            name = 'fire';
        case 5
            name ='plane';
        case 6 
            name ='shuttle';
        case 7
            name = 'spider';
        case 8
            name = 'square';
        case 9
            name ='star';
        case 10
            name ='tiger';
        case 11
            name='horse';
        case 12
            name='thunder';
end

%% Coordinates sequence of image
ImageName = [path, name, '.png'];
img=imread(ImageName);
if size(img,3)==3
    img = rgb2gray(imread(ImageName));
end
img=~im2bw(img,graythresh(img));
[M,N]=size(img);

[Edge, cross, OpenFlag]=traceContour(img);  
% traceContour known bug: only support contours with 1 pixel thick, 8-conntected neighboring
% alternative approach is to use code below for closed contour
% Edge =  bwtraceboundary(A2, P, 'E');
% cross=[];
% OpenFlag=zeros(1, length(Edge));

for i=1:length(Edge)
    edgey{i} = Edge{i}(:,1)'; % y coordinate of edge
    edgex{i} = Edge{i}(:,2)'; % x coordinate of edge
end

%% Encoding

% parameter for the DCT-based method
q=2;  % quantization factor
pn=3;   % the number of coefficients retaining for the DCT
eth=20; % error threshold

tic;
[Code, x_apx, y_apx, DPx, DPy] = DCT_encode(edgex, edgey, M, N, OpenFlag, cross, pn, eth, q);
encoding_time=toc;
disp(['Encoding time: ' num2str(encoding_time) 'seconds.']);


%% Decoding
tic
[x_apx_r, y_apx_r, DPx_r, DPy_r] = DCT_decode(Code, M, N, pn, q);    % _r for recovery
decoding_time=toc;
disp(['Decoding time: ' num2str(decoding_time) 'seconds.']);

%% Evaluation
[DP, ISE, MSE, CR] = evaluate2(DPx, DPy, x_apx_r, y_apx_r, edgex, edgey);
disp(['Number of DP: ' num2str(DP)]);
disp(['Integral Squared Error: ' num2str(ISE)]);
disp(['Mean Squared Error: ' num2str(MSE)]);
disp(['Compression Ratio: ' num2str(CR)]);
disp(['Coding Length: ' num2str(length(Code))]);

%% Plot
Nc=length(edgex);
figure('Units','normalized','position',[.2 .2 N/max(M,N)/2 M/max(M,N)/2]);
subplot(1,2,1); hold on
title('The Original Contour');
for i=1:Nc
    plot(edgex{i},edgey{i},'b');
end
set(gca,'Ydir','reverse','XTickLabel',[],'YTickLabel',[]); axis([1,N,1,M]);
% legend('Original Contour','location','best');
hold off

subplot(1,2,2); hold on
title('The Approximated Contour');
for i=1:Nc
    plot(x_apx_r{i},y_apx_r{i},'k',DPx_r{i}, DPy_r{i},'ro'); 
end
set(gca,'Ydir','reverse','XTickLabel',[],'YTickLabel',[]); axis([1,N,1,M]);
legend('Approximated Contour','Dominant Points','location','best');
hold off